# Do not change!
TAXURL = 'http://rest.kegg.jp/list/organism'
KOURL = 'http://rest.kegg.jp/link/ko/'
ECURL = 'http://rest.kegg.jp/link/enzyme/'

PWURL = 'http://rest.kegg.jp/link/pathway/'

ECURL = 'http://rest.kegg.jp/link/enzyme/'
ECURL = 'http://rest.kegg.jp/link/enzyme/'
ECURL = 'http://rest.kegg.jp/link/enzyme/'


