# gene name and description

# ncbi gene database
# wget https://ftp.ncbi.nih.gov/gene/DATA/gene_info.gz
gene_info_file = '/lustre/home/xuyuxing/Database/NCBI/gene/2020/gene_info.gz'

# wget https://ftp.ncbi.nih.gov/gene/DATA/gene2accession.gz
gene2accession = '/lustre/home/xuyuxing/Database/NCBI/gene/2020/gene2accession.gz'

